import { readFileSync } from "fs"
import { exit } from "process"
import { ZodError, z } from "zod"
export const defaultInterval = 60
export const defaultNotifyInterval = 300

const configSchema = z.object({
    defaults: z.object({
        retries: z.number().int().min(0).default(1),
        interval: z.number().default(30)
    }),
    alertEndpoints: z.array(z.object({
        enabled: z.boolean(),
        name: z.string()
    }).and(z.union([
        z.object({
            type: z.literal("discord-webhook"),
            webhookUrl: z.string()
        }),
        z.object({
            type: z.literal("telegram"),
            token: z.string(),
            chatIds: z.array(z.string())
        }),
        z.object({
            type: z.literal("pushover"),
            token: z.string(),
            destinationIds: z.array(z.string())
        })
    ]))),
    pollEndpoints: z.array(z.object({
        name: z.string(),
        interval: z.number().optional(),
        notifyInterval: z.number().default(300),
        alertEndpoints: z.array(z.string()),
        retries: z.number().int().min(0).optional()
    }).and(z.union([
        z.object({
            type: z.literal("fetch"),
            url: z.string()
        }),
        z.object({
            type: z.literal("ping"),
            address: z.string()
        }),
        z.object({
            type: z.literal("tcp-ping"),
            address: z.string(),
            port: z.number().default(80)
        })
    ])))
})

export type Config = z.infer<typeof configSchema>

const loadConfig = () => { 
    try { 
    return configSchema.parse(JSON.parse(readFileSync("./config.json").toString()))
    }
    catch(e) {
        const err = e as ZodError | SyntaxError
        if (err instanceof ZodError) {
            console.error(err.flatten())
        }
        else {
            console.error(err.message)
        }
        
        exit(1)
    }
}

export const config = loadConfig()